@@ -51,6 +51,7 @@ prepare_function <- function(x,
   select_model <- NULL;
   fixedsi <- NULL;
   match_sitar_d_form <- NULL;
+  match_sitar_a_form <- NULL;
   d_adjustedsi <- NULL;
   randomsi <- NULL;
   getxname <- NULL;
@@ -77,6 +78,16 @@ prepare_function <- function(x,
   }
   
   
+  # "X" for rcsfunmultadd 
+  
+  include_fun_c <- c(spfncname
+                     , 'getx' 
+                     , 'getknots'
+                     , "X"
+                     , 'd0'
+                     , 'd1' 
+                     # , 'd2'
+  )
   
   
   backend <- eval(brms_arguments$backend)
@@ -733,7 +744,8 @@ prepare_function <- function(x,
     
     
     if (select_model == 'sitar') {
-      vectorA <- "\n  vector[N] A=a-(s1*min(knots));"
+      if(match_sitar_a_form) vectorA <- "\n  vector[N] A=a-(s1*min(knots));"
+      if(!match_sitar_a_form) vectorA <- "\n  vector[N] A=a;"
       if (!is.null(decomp)) {
         if (decomp == 'QR') {
           vectorA <- "\n  vector[N] A=a;"
@@ -1514,11 +1526,62 @@ prepare_function <- function(x,
     )
     
     
-    if (utils::packageVersion('rstan') < 2.26) {
+    # rcsfunmultadd <- NULL
+    
+    
+    include_fun_names <- c(spfncname)
+    
+    if('d0' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "_d0"))
+      spl_d0 <- spl_d0
+    } else {
+      spl_d0 <- NULL
+    }
+    
+    if('d1' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "_d1"))
+      spl_d1 <- spl_d1
+    } else {
+      spl_d1 <- NULL
+    }
+    
+    if('d2' %in% include_fun_c) {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "_d2"))
+      spl_d2 <- spl_d2
+    } else {
+      spl_d2 <- NULL
+    }
+    
+    if('getx' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, getxname)
+      getxname <- getxname
+    } else {
+      getxname <- NULL
+    }
+    
+    if('getknots' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, getknotsname)
+      getknotsname <- getknotsname
+    } else {
+      getknotsname <- NULL
+    }
+    
+    
+    if('X' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "X"))
+      rcsfunmultadd <- rcsfunmultadd
+    } else {
+      rcsfunmultadd <- NULL
+    }
+    
+    
+    
+    
+    if (utils::packageVersion('rstan') < "2.26") {
       rcsfun <- paste(getx_knots_fun, rcsfun)
     }
     
-    if (utils::packageVersion('rstan') > 2.26 & is.null(decomp)) {
+    if (utils::packageVersion('rstan') > "2.26" & is.null(decomp)) {
       rcsfun <- paste0(getx_knots_fun,
                        rcsfun,
                        rcsfunmultadd,
@@ -1528,7 +1591,7 @@ prepare_function <- function(x,
                        sep = "\n")
     }
     
-    if (utils::packageVersion('rstan') > 2.26 & !is.null(decomp)) {
+    if (utils::packageVersion('rstan') > "2.26" & !is.null(decomp)) {
       if (decomp == 'QR') {
         if (add_funmats) {
           rcsfun <- paste0(
@@ -1623,7 +1686,7 @@ prepare_function <- function(x,
     
     if (select_model == 'pb1') {
       funstring <- "a-2.0*(a-b)./(exp(c.*(Xm-e))+exp(d.*(Xm-e)))"
-      if (utils::packageVersion('rstan') < 2.26)
+      if (utils::packageVersion('rstan') < "2.26")
         funstring <-
           gsub(".*", " .* ", funstring, fixed = T)
       returnmu    <- paste0("return ", "(",  funstring, ")")
@@ -1659,7 +1722,7 @@ prepare_function <- function(x,
     if (select_model == 'pb2') {
       funstring <- "a-((a-b)./(((0.5*exp((f.*c).*(Xm-e)))+
       (0.5*exp((f.*d).*(Xm-e))))^(1.0./f)))"
-      if (utils::packageVersion('rstan') < 2.26)
+      if (utils::packageVersion('rstan') < "2.26")
         funstring <-
           gsub(".*", " .* ", funstring, fixed = T)
       returnmu    <- paste0("return ", "(",  funstring, ")")
@@ -1752,7 +1815,7 @@ prepare_function <- function(x,
     if (select_model == 'pb3') {
       funstring <- "a-((4.0*(a-b))./((exp(f.*(Xm-e))+
       exp(c.*(Xm-e))).*(1.0+exp(d.*(Xm-e)))))"
-      if (utils::packageVersion('rstan') < 2.26)
+      if (utils::packageVersion('rstan') < "2.26")
         funstring <- gsub(".*", " .* ",
                           funstring,
                           fixed = T)
@@ -1827,7 +1890,7 @@ prepare_function <- function(x,
     
     if (select_model == 'logistic1') {
       funstring <- "a./(1+exp(-b.*(Xm-c)))"
-      if (utils::packageVersion('rstan') < 2.26)
+      if (utils::packageVersion('rstan') < "2.26")
         funstring <-
           gsub(".*", " .* ", funstring, fixed = T)
       returnmu    <- paste0("return ", "(",  funstring, ")")
@@ -1872,7 +1935,7 @@ prepare_function <- function(x,
     if (select_model == 'logistic2') {
       funstring <-
         "((a-b)./(1+exp(-e.*(Xm-f)))) + (b./(1+exp(-c.*(Xm-d))))"
-      if (utils::packageVersion('rstan') < 2.26)
+      if (utils::packageVersion('rstan') < "2.26")
         funstring <-
           gsub(".*", " .* ", funstring, fixed = T)
       returnmu    <- paste0("return ", "(",  funstring, ")")
@@ -1925,7 +1988,7 @@ prepare_function <- function(x,
         "(a ./ (1 + exp(-b .* (Xm - c)))) +
         (d ./ (1 + exp(-e .* (Xm - f)))) +
         (g ./ (1 + exp(-h .* (Xm - i))))"
-      if (utils::packageVersion('rstan') < 2.26)
+      if (utils::packageVersion('rstan') < "2.26")
         funstring <-
           gsub(".*", " .* ", funstring, fixed = T)
       returnmu    <- paste0("return ", "(",  funstring, ")")
@@ -2067,9 +2130,57 @@ prepare_function <- function(x,
     )
     
     
-    rcsfunmultadd <- NULL
+    # rcsfunmultadd <- NULL
     
-    if (utils::packageVersion('rstan') > 2.26 & is.null(decomp)) {
+    
+    include_fun_names <- c(spfncname)
+    
+    if('d0' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "_d0"))
+      spl_d0 <- spl_d0
+    } else {
+      spl_d0 <- NULL
+    }
+    
+    if('d1' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "_d1"))
+      spl_d1 <- spl_d1
+    } else {
+      spl_d1 <- NULL
+    }
+    
+    if('d2' %in% include_fun_c) {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "_d2"))
+      spl_d2 <- spl_d2
+    } else {
+      spl_d2 <- NULL
+    }
+    
+    if('getx' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, getxname)
+      getxname <- getxname
+    } else {
+      getxname <- NULL
+    }
+    
+    if('getknots' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, getknotsname)
+      getknotsname <- getknotsname
+    } else {
+      getknotsname <- NULL
+    }
+    
+    
+    if('X' %in% include_fun_c)  {
+      include_fun_names <- c(include_fun_names, paste0(spfncname, "X"))
+      rcsfunmultadd <- rcsfunmultadd
+    } else {
+      rcsfunmultadd <- NULL
+    }
+    
+    
+    
+    if (utils::packageVersion('rstan') > "2.26" & is.null(decomp)) {
       rcsfun <- paste0(getx_fun,
                        rcsfun,
                        rcsfunmultadd,
@@ -2079,7 +2190,7 @@ prepare_function <- function(x,
                        sep = "\n")
     }
     
-    if (utils::packageVersion('rstan') > 2.26 & !is.null(decomp)) {
+    if (utils::packageVersion('rstan') > "2.26" & !is.null(decomp)) {
       if (decomp == 'QR') {
         if (add_funmats) {
           rcsfun <- paste0(getx_fun,
@@ -2109,10 +2220,29 @@ prepare_function <- function(x,
   
   
   
+  # Remove empty lines from code strings
+  remove_spaces_and_tabs <- function(x) {
+    if(!is.null(x)) {
+      x <- gsub("^ *|(?<= ) | *$", "", x, perl = TRUE)
+      # '\\L\\1' converts first letter beyoind .* to lower
+      # x <- gsub("(\\..*?[A-Z]|^[A-Z])", '\\L\\1', x, perl=T)
+      x <- gsub("(\\..*?[A-Z]|^[A-Z])", '\\1', x, perl=T)
+      x <- x[x != ""]
+      x <- gsub("\\s*\n\\s*","\n",x) 
+      xx <- x
+    } else {
+      xx <- x
+    }
+    return(xx)
+  }
+  
+  
+  
   
   #################
   extract_r_fun_from_scode <-
     function(xstaring, what = NULL, decomp, spfncname) {
+      if(is.null(xstaring)) return(xstaring)
       xstaring <- gsub("[[:space:]]" , "", xstaring)
       xstaring <- gsub(";" , ";\n", xstaring)
       xstaring <- gsub("\\{" , "{\n", xstaring)
@@ -2197,7 +2327,9 @@ prepare_function <- function(x,
                  fixed = T)
         }
       }
-      
+      xstaring <- gsub("matrixXp" , "Xp", xstaring, fixed = T) # spfnameX
+      xstaring <- gsub("mcolsmat=cols(Xp);" , "", xstaring, fixed = T) # spfnameX
+      xstaring <- gsub("[mcolsmat+1]" , "", xstaring, fixed = T) # spfnameX
       xstaring
     } # extract_r_fun_from_scode
   
@@ -2222,8 +2354,8 @@ prepare_function <- function(x,
     getx_fun_raw,
     what = 'getX',
     decomp = decomp,
-    spfncname = spfncname
-  )
+    spfncname = spfncname)
+  
   getknots_str <- NULL
   if (select_model == 'sitar' | select_model == 'rcs') {
     getknots_str <- extract_r_fun_from_scode(
@@ -2234,22 +2366,37 @@ prepare_function <- function(x,
     )
   }
   
+  
+  rcsfunmultadd_str     <- extract_r_fun_from_scode(
+    rcsfunmultadd,
+    what = 'X',
+    decomp = decomp,
+    spfncname = spfncname)
+  
+  
+  
   all_raw_str <- c(rcsfun_raw_str,
                    spl_d0_str,
                    spl_d1_str,
                    spl_d2_str,
                    getX_str,
-                   getknots_str)
+                   getknots_str,
+                   rcsfunmultadd_str)
+  
+  rcsfun <- remove_spaces_and_tabs(rcsfun)
   
   
   if (!add_rcsfunmatqrinv_genquant) {
-    out <- list(rcsfun = rcsfun, r_funs = all_raw_str)
+    out <- list(rcsfun = rcsfun, r_funs = all_raw_str,
+                include_fun_names = include_fun_names)
   } else if (add_rcsfunmatqrinv_genquant) {
     out <- list(rcsfun = rcsfun,
                 r_funs = all_raw_str,
-                gq_funs = rcsfunmatqrinv_genquant)
+                gq_funs = rcsfunmatqrinv_genquant,
+                include_fun_names = include_fun_names)
   }
   
+  
   out
 }
 
